$title  A Transportation Problem (TRNSPORT,SEQ=1)

$onText
This problem finds a least cost shipping schedule that meets
requirements at markets and supplies at factories.


Dantzig, G B, Chapter 3.3. In Linear Programming and Extensions.
Princeton University Press, Princeton, New Jersey, 1963.

This formulation is described in detail in:
Rosenthal, R E, Chapter 2: A GAMS Tutorial. In GAMS: A User's Guide.
The Scientific Press, Redwood City, California, 1988.

The line numbers will not match those in the book because of these
comments.

Keywords: linear programming, transportation problem, scheduling
$offText

Set
   i 'canning plants' / seattle,  san-diego /
   j 'markets'        / new-york, chicago, topeka /;

Parameter
   a(i) 'capacity of plant i in cases'
        / seattle     350
          san-diego   600  /

   b(j) 'demand at market j in cases'
        / new-york    325
          chicago     300
          topeka      275  /;

Table d(i,j) 'distance in thousands of miles'
               new-york       chicago        topeka
   seattle          2.5           1.7           1.8
   san-diego        2.5           1.8           1.4;

Scalar f 'freight in dollars per case per thousand miles' / 90 /;

Parameter c(i,j) 'transport cost in thousands of dollars per case';
c(i,j) = f * d(i,j) / 1000;

Variable
   x(i,j) 'shipment quantities in cases'
   z      'total transportation costs in thousands of dollars';

Positive Variable x;

Equation
   cost      'define objective function'
   supply(i) 'observe supply limit at plant i'
   demand(j) 'satisfy demand at market j';

cost..        z  =e=  sum((i,j), c(i,j)*x(i,j));

supply(i)..   sum(j, x(i,j)) =l= a(i);

demand(j)..   sum(i, x(i,j)) =g= b(j);

Model transport / all /;

set allcases /base,case1*case3/;
set cases(allcases) /base,case1/;
set sd /supply,demand/;
table changetomake(allcases,sd) data to change by case - note make sure to keep supply ge than demand
             supply  demand
base
case1          1.1
case2          1.05    1.01
case3                  0.90;
parameter  saved_a(i),
           saved_b(j),
           sv_x_sol(i,j,allcases)
           sv_supply_shadprice(i,allcases)
           sv_demand_shadprice(j,allcases)
           sv_solution_status(*,allcases)
           supplychange
           demandchange;

           saved_a(i)=a(i);
           saved_b(j)=b(j);
option solprint=off;
option limrow=0;
option limcol=0;
*this the gams i am trying to replicate
loop(cases,
    supplychange=1;
    supplychange$changetomake(cases,"supply")=changetomake(cases,"supply");
    demandchange=1;
    demandchange$changetomake(cases,"demand")=changetomake(cases,"demand");
    a(i)=saved_a(i)*supplychange;
    b(j)=saved_b(j)*demandchange;
    solve transport using lp minimizing z;
    sv_x_sol(i,j,cases)=x.l(i,j);
    sv_supply_shadprice(i,cases)=supply.m(i);
    sv_demand_shadprice(j,cases)=demand.m(j);
    sv_solution_status("Obj Value",cases)=z.l;
    sv_solution_status("Solvestat",cases)=transport.solvestat;
    sv_solution_status("Modelstat",cases)=transport.modelstat;
    sv_solution_status("Solve_time",cases)=transport.etsolver;
    sv_solution_status("Number_infeasibilities",cases)=transport.numInfes;
);
execute_unload 'gamsloop', sv_x_sol,sv_supply_shadprice,sv_demand_shadprice,sv_solution_status;

* Now the same stuff with input modification and result collection in Python
embeddedCode Python:
storeallsupplymarg      = []
storealldemandmarg      = []
storeallxval            = []
storeall_solution_stuff = []

savea        = list(gams.get('saved_a'))
saveb        = list(gams.get('saved_b'))
changetomake = dict(gams.get('changetomake'))
pauseembeddedcode

*this the gams i am trying to replicate
singleton set case(cases) /base/;
parameter solution_stuff(*);
loop(cases,
    case(cases)=yes;
*   now change the data using python code
    continueembeddedCode:
    # get the data from GAMS
    case         = list(gams.get('case'))[0]
    supplychange = changetomake.get((case,'supply'),1) # if the record is not there use default value 1
    demandchange = changetomake.get((case,'demand'),1) # if the record is not there use default value 1

    # multiply the rhs data by the constant
    a = [(i[0],supplychange*i[1]) for i in savea]
    b = [(i[0],demandchange*i[1]) for i in saveb]
    gams.set("a",a)
    gams.set("b",b)
    pauseEmbeddedCode a b
    display a,b;

    option solvelink=2;
    solve transport using lp minimizing z;
    solution_stuff("Obj Value")=z.l;
    solution_stuff("Solvestat")=transport.solvestat;
    solution_stuff("Modelstat")=transport.modelstat;
    solution_stuff("Solve_time")=transport.etsolver;
    solution_stuff("Number_infeasibilities")=transport.numInfes;

*   now back to python to build saved data arrays
    continueEmbeddedCode:
    # get solution data from gams
    xvalue = list(gams.get('x'))
    for irec in gams.get('supply'):
      storeallsupplymarg.append((irec[0],case,irec[1][GMS_VAL_MARGINAL]))
    for jrec in gams.get('demand'):
      storealldemandmarg.append((jrec[0],case,jrec[1][GMS_VAL_MARGINAL]))
    for ijrec in gams.get('x'):
      storeallxval.append((*ijrec[0],case,ijrec[1][GMS_VAL_LEVEL]))
    for mrec in gams.get('solution_stuff'):
      storeall_solution_stuff.append((mrec[0],case,mrec[1]))
    pauseEmbeddedCode
);

* Now bring the reports back to GAMS
continueEmbeddedCode:
gams.set("sv_x_sol",storeallxval)
gams.set("sv_supply_shadprice",storeallsupplymarg)
gams.set("sv_demand_shadprice",storealldemandmarg)
gams.set("sv_solution_status",storeall_solution_stuff)
endEmbeddedCode sv_x_sol sv_supply_shadprice sv_demand_shadprice sv_solution_status


execute_unload 'pyloop', sv_x_sol,sv_supply_shadprice,sv_demand_shadprice,sv_solution_status;
execute 'gdxdiff gamsloop.gdx pyloop.gdx';
* Test to see if they are the same
*This wont always come out clean as the solution time and shadow prices (EPS) may differ
*abort$errorlevel 'GDX files are different!'

$exit
*then a solve
$ontext
continueEmbeddedCode

get solution data from gams

  recoverobj = list(gams.get('z'))
  print (recoverobj)
*try to get model attricultres from gams
#  recoverattributes = list(gams.get('transport'))
#  print (recoverattributes)
*put solution into a vector
  for j in range(len(recoverdemandmarg1)):
      recoverdemandmarg.append([recoverdemandmarg1[j][0],recoverdemandmarg1[j][1][1]])
  for i in range(len(recoversupplymarg1)):
      recoversupplymarg.append([recoversupplymarg1[i][0],recoversupplymarg1[i][1][1]])
  for i in range(len(recoverxvalue)):
      recoveroptx.append([recoverxvalue[i][0],recoverxvalue[i][1][0]])
  print (recoveroptx,recoverxvalue)

now i would like to build a table that has the locational substripts and case names then put this back into gams

$offtext

display saved_x_solution;
